package com.javahonk.jms.solace;

import javax.annotation.PostConstruct;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.naming.InitialContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import com.solacesystems.jms.SolJmsUtility;
import com.solacesystems.jms.message.SolBytesMessage;
import com.solacesystems.jms.message.SolTextMessage;

public class SolaceMessageSubscriber {
	
	private static final Logger logger = LogManager.getLogger(SolaceMessageSubscriber.class.getName());

	@Autowired
    private InitialContext initialContext_subscriber;
	
	@Value("${SOLACE_CONNECTION_FACTORY}")
	private String solaceConnectionFactoryLookup;
	
	@Value("${SOLACE_OUTPUT_QUEUE_NAME}")
	private String solaceOutputQueueName;
	
	private MessageConsumer consumer;
	
	@PostConstruct
	public void init() {
		
		logger.info("Solace outputQueue: {}", solaceOutputQueueName);
		
		try {
			
			QueueConnectionFactory cf = (QueueConnectionFactory)initialContext_subscriber.lookup(solaceConnectionFactoryLookup);
	    	QueueConnection connection = cf.createQueueConnection();
	        Session session = connection.createQueueSession(false, Session.CLIENT_ACKNOWLEDGE);
	        Queue queue = (Queue)initialContext_subscriber.lookup(solaceOutputQueueName);
	        consumer = session.createConsumer(queue);	        
	        
	        connection.start();	        
	        onReceive();
	        
	        logger.info("Connection made successfully. Waiting for a message...");
			
		} catch (Exception e) {
			logger.error("Error creating connection to Solace: {}", e);
		} 
	}
	
	public void onReceive() {		
		
		try {
			
			while (true) {
	            
				Message testMessage = consumer.receive();
				
				logger.info("Message received");
	            
	            if (testMessage == null) {	            	
	            	logger.info("An error has occured. Solace Text message was null");	            	
	            }
	            
	            if (testMessage instanceof SolBytesMessage) {
	            	
	            	System.out.println("Message received:"+((SolBytesMessage)testMessage).getMessage());
	            	logger.info("Message size: {} kb",SolJmsUtility.getMessageSize(testMessage));
	            	testMessage.acknowledge();	                 
	            }
	            
	            if (testMessage instanceof SolTextMessage) {
	            	
	            	System.out.println("Message received:"+((SolTextMessage)testMessage).getText());
	            	logger.info("Message size: {} kb",SolJmsUtility.getMessageSize(testMessage));
	            	testMessage.acknowledge();
	            }
	        }
			
		} catch (Exception e) {
			logger.error("Error sending message to output queue name:{} --> details exceptions: --> ", solaceOutputQueueName, e);
		}
		
	}

}
